summaryrefslogtreecommitdiff
diff options
context:
space:
mode:
authorChris Lattner <sabre@nondot.org>2001-10-13 07:01:45 +0000
committerChris Lattner <sabre@nondot.org>2001-10-13 07:01:45 +0000
commit49ad898e3314771e9a87290ad795f31ce78e1f2d (patch)
tree683bb800cdeaffd0a81bba0f20f43cf6a070cb95
parent56c5acbe9600f33a3d7260ef68211203cf675279 (diff)
downloadllvm-49ad898e3314771e9a87290ad795f31ce78e1f2d.tar.gz
llvm-49ad898e3314771e9a87290ad795f31ce78e1f2d.tar.bz2
llvm-49ad898e3314771e9a87290ad795f31ce78e1f2d.tar.xz
Implement the invoke instruction
git-svn-id: https://llvm.org/svn/llvm-project/llvm/trunk@774 91177308-0d34-0410-b5e6-96231b3b80d8
-rw-r--r--lib/VMCore/iCall.cpp61
1 files changed, 50 insertions, 11 deletions
diff --git a/lib/VMCore/iCall.cpp b/lib/VMCore/iCall.cpp
index f739730b67..fc718d6887 100644
--- a/lib/VMCore/iCall.cpp
+++ b/lib/VMCore/iCall.cpp
@@ -1,27 +1,33 @@
//===-- iCall.cpp - Implement the call & icall instructions ------*- C++ -*--=//
//
// This file implements the call and icall instructions.
//
//===----------------------------------------------------------------------===//
#include "llvm/iOther.h"
+#include "llvm/iTerminators.h"
#include "llvm/DerivedTypes.h"
#include "llvm/Method.h"
-CallInst::CallInst(Method *M, const vector<Value*> &params,
- const string &Name)
- : Instruction(M->getReturnType(), Instruction::Call, Name) {
+//===----------------------------------------------------------------------===//
+// CallInst Implementation
+//===----------------------------------------------------------------------===//
+CallInst::CallInst(Value *Meth, const vector<Value*> &params,
+ const string &Name)
+ : Instruction(cast<MethodType>(cast<PointerType>(Meth->getType())
+ ->getValueType())->getReturnType(),
+ Instruction::Call, Name) {
Operands.reserve(1+params.size());
- Operands.push_back(Use(M, this));
+ Operands.push_back(Use(Meth, this));
+
+ const MethodType *MTy =
+ cast<MethodType>(cast<PointerType>(Meth->getType())->getValueType());
- const MethodType::ParamTypes &PL = M->getMethodType()->getParamTypes();
+ const MethodType::ParamTypes &PL = MTy->getParamTypes();
assert((params.size() == PL.size()) ||
- (M->getMethodType()->isVarArg() && params.size() > PL.size()) &&
+ (MTy->isVarArg() && params.size() >= PL.size()-1) &&
"Calling a function with bad signature");
-#ifndef NDEBUG
- MethodType::ParamTypes::const_iterator It = PL.begin();
-#endif
for (unsigned i = 0; i < params.size(); i++)
Operands.push_back(Use(params[i], this));
}
@@ -33,3 +39,36 @@ CallInst::CallInst(const CallInst &CI)
Operands.push_back(Use(CI.Operands[i], this));
}
+//===----------------------------------------------------------------------===//
+// InvokeInst Implementation
+//===----------------------------------------------------------------------===//
+
+InvokeInst::InvokeInst(Value *Meth, BasicBlock *IfNormal, \
+ BasicBlock *IfException, const vector<Value*>&params,
+ const string &Name)
+ : TerminatorInst(cast<MethodType>(cast<PointerType>(Meth->getType())
+ ->getValueType())->getReturnType(),
+ Instruction::Invoke, Name) {
+ Operands.reserve(3+params.size());
+ Operands.push_back(Use(Meth, this));
+ Operands.push_back(Use(IfNormal, this));
+ Operands.push_back(Use(IfException, this));
+ const MethodType *MTy =
+ cast<MethodType>(cast<PointerType>(Meth->getType())->getValueType());
+
+ const MethodType::ParamTypes &PL = MTy->getParamTypes();
+ assert((params.size() == PL.size()) ||
+ (MTy->isVarArg() && params.size() > PL.size()) &&
+ "Calling a function with bad signature");
+
+ for (unsigned i = 0; i < params.size(); i++)
+ Operands.push_back(Use(params[i], this));
+}
+
+InvokeInst::InvokeInst(const InvokeInst &CI)
+ : TerminatorInst(CI.getType(), Instruction::Invoke) {
+ Operands.reserve(CI.Operands.size());
+ for (unsigned i = 0; i < CI.Operands.size(); ++i)
+ Operands.push_back(Use(CI.Operands[i], this));
+}
+
@@ -1,27 +1,33 @@
-//===-- iCall.cpp - Implement the call & icall instructions ------*- C++ -*--=//
+//===-- iCall.cpp - Implement the call & invoke instructions -----*- C++ -*--=//
//
@@ -1,27 +1,33 @@
//
-// This file implements the call and icall instructions.
+// This file implements the call and invoke instructions.
//
//===----------------------------------------------------------------------===//